@@ -1,28 +1,50 @@
+# -*- coding: utf-8 -*-
 import os
 import time
-from itertools import chain
 
 from nose.plugins.skip import SkipTest
 
 from pydocx.tests.document_builder import DocxBuilder as DXB
 from pydocx.tests import (
-    ElementTree,
     XMLDocx2Html,
     _TranslationTestCase,
-    remove_namespaces,
 )
+from pydocx.utils import parse_xml_from_string, find_all
 
 
 class BoldTestCase(_TranslationTestCase):
     expected_output = """
         <p><strong>AAA</strong></p>
         <p>BBB</p>
+        <p>CCC</p>
     """
 
     def get_xml(self):
         tags = [
-            DXB.p_tag(text='AAA', bold=True),
-            DXB.p_tag(text='BBB', bold=True, val='false'),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('AAA')],
+                        rpr=DXB.rpr_tag({'b': None}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('BBB')],
+                        rpr=DXB.rpr_tag({'b': 'false'}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('CCC')],
+                        rpr=DXB.rpr_tag({'b': '0'}),
+                    ),
+                ],
+            ),
         ]
 
         body = ''
@@ -33,6 +55,7 @@ class BoldTestCase(_TranslationTestCase):
 
 
 class HyperlinkVanillaTestCase(_TranslationTestCase):
+
     relationship_dict = {
         'rId0': 'www.google.com',
     }
@@ -43,9 +66,9 @@ class HyperlinkVanillaTestCase(_TranslationTestCase):
 
     def get_xml(self):
         run_tags = []
-        run_tags.append(DXB.r_tag('link', is_bold=False))
+        run_tags.append(DXB.r_tag([DXB.t_tag('link')]))
         run_tags = [DXB.hyperlink_tag(r_id='rId0', run_tags=run_tags)]
-        run_tags.append(DXB.r_tag('.', is_bold=False))
+        run_tags.append(DXB.r_tag([DXB.t_tag('.')]))
         body = DXB.p_tag(run_tags)
         xml = DXB.xml(body)
         return xml
@@ -61,9 +84,9 @@ class HyperlinkWithMultipleRunsTestCase(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        run_tags = [DXB.r_tag(i) for i in 'link']
+        run_tags = [DXB.r_tag([DXB.t_tag(i)]) for i in 'link']
         run_tags = [DXB.hyperlink_tag(r_id='rId0', run_tags=run_tags)]
-        run_tags.append(DXB.r_tag('.', is_bold=False))
+        run_tags.append(DXB.r_tag([DXB.t_tag('.')]))
         body = DXB.p_tag(run_tags)
         xml = DXB.xml(body)
         return xml
@@ -93,9 +116,9 @@ class HyperlinkNotInRelsDictTestCase(_TranslationTestCase):
 
     def get_xml(self):
         run_tags = []
-        run_tags.append(DXB.r_tag('link', is_bold=False))
+        run_tags.append(DXB.r_tag([DXB.t_tag('link')]))
         run_tags = [DXB.hyperlink_tag(r_id='rId0', run_tags=run_tags)]
-        run_tags.append(DXB.r_tag('.', is_bold=False))
+        run_tags.append(DXB.r_tag([DXB.t_tag('.')]))
         body = DXB.p_tag(run_tags)
         xml = DXB.xml(body)
         return xml
@@ -110,8 +133,8 @@ class HyperlinkWithBreakTestCase(_TranslationTestCase):
 
     def get_xml(self):
         run_tags = []
-        run_tags.append(DXB.r_tag('link'))
-        run_tags.append(DXB.r_tag(None, include_linebreak=True))
+        run_tags.append(DXB.r_tag([DXB.t_tag('link')]))
+        run_tags.append(DXB.r_tag([DXB.linebreak()]))
         run_tags = [DXB.hyperlink_tag(r_id='rId0', run_tags=run_tags)]
         body = DXB.p_tag(run_tags)
         xml = DXB.xml(body)
@@ -176,12 +199,10 @@ class ImageTestCase(_TranslationTestCase):
             document_xml=self.get_xml(),
             rels_dict=self.relationship_dict,
         )
-        tree = ElementTree.fromstring(
-            remove_namespaces(self.get_xml()),
-        )
+        tree = parse_xml_from_string(self.get_xml())
         els = []
-        els.extend(tree.find_all('drawing'))
-        els.extend(tree.find_all('pict'))
+        els.extend(find_all(tree, 'drawing'))
+        els.extend(find_all(tree, 'pict'))
         image_ids = []
         for el in els:
             image_ids.append(parser._get_image_id(el))
@@ -199,12 +220,10 @@ class ImageTestCase(_TranslationTestCase):
             document_xml=self.get_xml(),
             rels_dict=self.relationship_dict,
         )
-        tree = ElementTree.fromstring(
-            remove_namespaces(self.get_xml()),
-        )
+        tree = parse_xml_from_string(self.get_xml())
         els = []
-        els.extend(tree.find_all('drawing'))
-        els.extend(tree.find_all('pict'))
+        els.extend(find_all(tree, 'drawing'))
+        els.extend(find_all(tree, 'pict'))
         image_ids = []
         for el in els:
             image_ids.append(parser._get_image_size(el))
@@ -287,12 +306,40 @@ class TableTag(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        table = DXB.table(num_rows=2, num_columns=2, text=chain(
-            [DXB.p_tag('AAA')],
-            [DXB.p_tag('BBB')],
-            [DXB.p_tag('CCC')],
-            [DXB.p_tag('DDD')],
-        ))
+        cell1 = DXB.table_cell(paragraph=DXB.p_tag('AAA'))
+        cell2 = DXB.table_cell(paragraph=DXB.p_tag('CCC'))
+        cell3 = DXB.table_cell(paragraph=DXB.p_tag('BBB'))
+        cell4 = DXB.table_cell(paragraph=DXB.p_tag('DDD'))
+        rows = [DXB.table_row([cell1, cell3]), DXB.table_row([cell2, cell4])]
+        table = DXB.table(rows)
+        body = table
+        xml = DXB.xml(body)
+        return xml
+
+
+class RowSpanTestCase(_TranslationTestCase):
+
+    expected_output = '''
+           <table border="1">
+            <tr>
+                <td rowspan="2">AAA</td>
+                <td>BBB</td>
+            </tr>
+            <tr>
+                <td>CCC</td>
+            </tr>
+        </table>
+    '''
+
+    def get_xml(self):
+        cell1 = DXB.table_cell(
+            paragraph=DXB.p_tag('AAA'), merge=True, merge_continue=False)
+        cell2 = DXB.table_cell(
+            paragraph=DXB.p_tag(None), merge=False, merge_continue=True)
+        cell3 = DXB.table_cell(paragraph=DXB.p_tag('BBB'))
+        cell4 = DXB.table_cell(paragraph=DXB.p_tag('CCC'))
+        rows = [DXB.table_row([cell1, cell3]), DXB.table_row([cell2, cell4])]
+        table = DXB.table(rows)
         body = table
         xml = DXB.xml(body)
         return xml
@@ -324,18 +371,18 @@ class NestedTableTag(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        nested_table = DXB.table(num_rows=2, num_columns=2, text=chain(
-            [DXB.p_tag('DDD')],
-            [DXB.p_tag('EEE')],
-            [DXB.p_tag('FFF')],
-            [DXB.p_tag('GGG')],
-        ))
-        table = DXB.table(num_rows=2, num_columns=2, text=chain(
-            [DXB.p_tag('AAA')],
-            [DXB.p_tag('BBB')],
-            [DXB.p_tag('CCC')],
-            [nested_table],
-        ))
+        cell1 = DXB.table_cell(paragraph=DXB.p_tag('DDD'))
+        cell2 = DXB.table_cell(paragraph=DXB.p_tag('FFF'))
+        cell3 = DXB.table_cell(paragraph=DXB.p_tag('EEE'))
+        cell4 = DXB.table_cell(paragraph=DXB.p_tag('GGG'))
+        rows = [DXB.table_row([cell1, cell3]), DXB.table_row([cell2, cell4])]
+        nested_table = DXB.table(rows)
+        cell1 = DXB.table_cell(paragraph=DXB.p_tag('AAA'))
+        cell2 = DXB.table_cell(paragraph=DXB.p_tag('CCC'))
+        cell3 = DXB.table_cell(paragraph=DXB.p_tag('BBB'))
+        cell4 = DXB.table_cell(nested_table)
+        rows = [DXB.table_row([cell1, cell3]), DXB.table_row([cell2, cell4])]
+        table = DXB.table(rows)
         body = table
         xml = DXB.xml(body)
         return xml
@@ -356,14 +403,12 @@ class TableWithInvalidTag(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        table = DXB.table(num_rows=2, num_columns=2, text=chain(
-            [DXB.p_tag('AAA')],
-            [DXB.p_tag('BBB')],
-            # This tag may have CCC in it, however this tag has no meaning
-            # pertaining to content.
-            ['<w:invalidTag>CCC</w:invalidTag>'],
-            [DXB.p_tag('DDD')],
-        ))
+        cell1 = DXB.table_cell(paragraph=DXB.p_tag('AAA'))
+        cell2 = DXB.table_cell('<w:invalidTag>CCC</w:invalidTag>')
+        cell3 = DXB.table_cell(paragraph=DXB.p_tag('BBB'))
+        cell4 = DXB.table_cell(paragraph=DXB.p_tag('DDD'))
+        rows = [DXB.table_row([cell1, cell3]), DXB.table_row([cell2, cell4])]
+        table = DXB.table(rows)
         body = table
         xml = DXB.xml(body)
         return xml
@@ -401,9 +446,9 @@ class TableWithListAndParagraph(_TranslationTestCase):
         td = ''
         for el in els:
             td += el
-        table = DXB.table(num_rows=1, num_columns=1, text=chain(
-            [td],
-        ))
+        cell1 = DXB.table_cell(td)
+        row = DXB.table_row([cell1])
+        table = DXB.table([row])
         body = table
         xml = DXB.xml(body)
         return xml
@@ -486,12 +531,12 @@ class ListWithContinuationTestCase(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        table = DXB.table(num_rows=2, num_columns=2, text=chain(
-            [DXB.p_tag('DDD')],
-            [DXB.p_tag('EEE')],
-            [DXB.p_tag('FFF')],
-            [DXB.p_tag('GGG')],
-        ))
+        cell1 = DXB.table_cell(paragraph=DXB.p_tag('DDD'))
+        cell2 = DXB.table_cell(paragraph=DXB.p_tag('FFF'))
+        cell3 = DXB.table_cell(paragraph=DXB.p_tag('EEE'))
+        cell4 = DXB.table_cell(paragraph=DXB.p_tag('GGG'))
+        rows = [DXB.table_row([cell1, cell3]), DXB.table_row([cell2, cell4])]
+        table = DXB.table(rows)
         tags = [
             DXB.li(text='AAA', ilvl=0, numId=1),
             DXB.p_tag('BBB'),
@@ -527,12 +572,12 @@ class ListWithMultipleContinuationTestCase(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        table1 = DXB.table(num_rows=1, num_columns=1, text=chain(
-            [DXB.p_tag('BBB')],
-        ))
-        table2 = DXB.table(num_rows=1, num_columns=1, text=chain(
-            [DXB.p_tag('CCC')],
-        ))
+        cell = DXB.table_cell(paragraph=DXB.p_tag('BBB'))
+        row = DXB.table_row([cell])
+        table1 = DXB.table([row])
+        cell = DXB.table_cell(paragraph=DXB.p_tag('CCC'))
+        row = DXB.table_row([cell])
+        table2 = DXB.table([row])
         tags = [
             DXB.li(text='AAA', ilvl=0, numId=1),
             table1,
@@ -639,19 +684,17 @@ class DeeplyNestedTableTestCase(_TranslationTestCase):
     run_expected_output = False
 
     def get_xml(self):
-        table = DXB.p_tag('AAA')
+        paragraph = DXB.p_tag('AAA')
 
         for _ in range(50):
-            table = DXB.table(num_rows=1, num_columns=1, text=chain(
-                [table],
-            ))
+            cell = DXB.table_cell(paragraph)
+            row = DXB.table_cell([cell])
+            table = DXB.table([row])
         body = table
         xml = DXB.xml(body)
         return xml
 
     def test_performance(self):
-        if not os.environ.get('TRAVIS_EXECUTE_PERFORMANCE', False):
-            raise SkipTest('TRAVIS_EXECUTE_PERFORMANCE is false')
         with self.toggle_run_expected_output():
             start_time = time.time()
             try:
@@ -661,7 +704,7 @@ class DeeplyNestedTableTestCase(_TranslationTestCase):
             end_time = time.time()
             total_time = end_time - start_time
             # This finishes in under a second on python 2.7
-            assert total_time < 5, total_time
+            assert total_time < 3, total_time
 
 
 class NonStandardTextTagsTestCase(_TranslationTestCase):
@@ -671,9 +714,9 @@ class NonStandardTextTagsTestCase(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        run_tags = [DXB.r_tag(i) for i in 'insert ']
+        run_tags = [DXB.r_tag([DXB.t_tag(i)]) for i in 'insert ']
         insert_tag = DXB.insert_tag(run_tags)
-        run_tags = [DXB.r_tag(i) for i in 'smarttag']
+        run_tags = [DXB.r_tag([DXB.t_tag(i)]) for i in 'smarttag']
         smart_tag = DXB.smart_tag(run_tags)
 
         run_tags = [insert_tag, smart_tag]
@@ -728,7 +771,7 @@ class InsertTagInList(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        run_tags = [DXB.r_tag(i) for i in 'BBB']
+        run_tags = [DXB.r_tag([DXB.t_tag(i)]) for i in 'BBB']
         insert_tags = DXB.insert_tag(run_tags)
         p_tag = DXB.p_tag([insert_tags])
 
@@ -750,7 +793,7 @@ class SmartTagInList(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        run_tags = [DXB.r_tag(i) for i in 'BBB']
+        run_tags = [DXB.r_tag([DXB.t_tag(i)]) for i in 'BBB']
         smart_tag = DXB.smart_tag(run_tags)
         p_tag = DXB.p_tag([smart_tag])
 
@@ -809,19 +852,20 @@ class SimpleTableTest(_TranslationTestCase):
         </table>'''
 
     def get_xml(self):
-        table = DXB.table(num_rows=3, num_columns=3, text=chain(
-            [DXB.p_tag('Blank')],
-            [DXB.p_tag('Column 1')],
-            [DXB.p_tag('Column 2')],
-            [DXB.p_tag('Row 1')],
-            [DXB.p_tag('First')],
-            [DXB.p_tag('Second')],
-            [DXB.p_tag('Row 2')],
-            [DXB.p_tag('Third')],
-            [DXB.p_tag('Fourth')],
-        ), merge=True)
+        cell1 = DXB.table_cell(paragraph=DXB.p_tag('Blank'))
+        cell2 = DXB.table_cell(paragraph=DXB.p_tag('Row 1'))
+        cell3 = DXB.table_cell(paragraph=DXB.p_tag('Row 2'))
+        cell4 = DXB.table_cell(paragraph=DXB.p_tag('Column 1'))
+        cell5 = DXB.table_cell(paragraph=DXB.p_tag('First'))
+        cell6 = DXB.table_cell(paragraph=DXB.p_tag('Third'))
+        cell7 = DXB.table_cell(paragraph=DXB.p_tag('Column 2'))
+        cell8 = DXB.table_cell(paragraph=DXB.p_tag('Second'))
+        cell9 = DXB.table_cell(paragraph=DXB.p_tag('Fourth'))
+        rows = [DXB.table_row([cell1, cell4, cell7]),
+                DXB.table_row([cell2, cell5, cell8]),
+                DXB.table_row([cell3, cell6, cell9])]
+        table = DXB.table(rows)
         body = table
-
         xml = DXB.xml(body)
         return xml
 
@@ -882,15 +926,14 @@ class SameNumIdInTable(_TranslationTestCase):
         lis = ''
         for text, ilvl, numId in li_text:
             lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
-        table = DXB.table(num_rows=1, num_columns=1, text=chain(
-            [lis],
-        ))
+        cell1 = DXB.table_cell(lis)
+        rows = DXB.table_row([cell1])
+        table = DXB.table([rows])
         lis = ''
         lis += DXB.li(text='AAA', ilvl=0, numId=1)
         lis += table
         lis += DXB.li(text='CCC', ilvl=0, numId=1)
         body = lis
-
         xml = DXB.xml(body)
         return xml
 
@@ -924,6 +967,7 @@ class HeadingTestCase(_TranslationTestCase):
         <h6>GGG</h6>
         <p>HHH</p>
     '''
+
     styles_dict = {
         'style0': 'heading 1',
         'style1': 'heading 2',
@@ -949,3 +993,263 @@ class HeadingTestCase(_TranslationTestCase):
 
         xml = DXB.xml(body)
         return xml
+
+
+class RomanNumeralToHeadingTestCase(_TranslationTestCase):
+    convert_root_level_upper_roman = True
+    numbering_dict = {
+        '1': {
+            '0': 'upperRoman',
+            '1': 'decimal',
+            '2': 'upperRoman',
+        },
+        '2': {
+            '0': 'upperRoman',
+            '1': 'decimal',
+            '2': 'upperRoman',
+        },
+        '3': {
+            '0': 'upperRoman',
+            '1': 'decimal',
+            '2': 'upperRoman',
+        },
+    }
+    expected_output = '''
+        <h2>AAA</h2>
+        <ol list-style-type="decimal">
+            <li>BBB</li>
+        </ol>
+        <h2>CCC</h2>
+        <ol list-style-type="decimal">
+            <li>DDD</li>
+        </ol>
+        <h2>EEE</h2>
+        <ol list-style-type="decimal">
+            <li>FFF
+                <ol list-style-type="upperRoman">
+                    <li>GGG</li>
+                </ol>
+            </li>
+        </ol>
+    '''
+
+    def get_xml(self):
+        li_text = [
+            ('AAA', 0, 1),
+            ('BBB', 1, 1),
+            ('CCC', 0, 2),
+            ('DDD', 1, 2),
+            ('EEE', 0, 3),
+            ('FFF', 1, 3),
+            ('GGG', 2, 3),
+        ]
+        body = ''
+        for text, ilvl, numId in li_text:
+            body += DXB.li(text=text, ilvl=ilvl, numId=numId)
+
+        xml = DXB.xml(body)
+        return xml
+
+
+class MultipleTTagsInRTag(_TranslationTestCase):
+    expected_output = '''
+        <p>ABC</p>
+    '''
+
+    def get_xml(self):
+        r_tag = DXB.r_tag(
+            [DXB.t_tag(letter) for letter in 'ABC'],
+        )
+        p_tag = DXB.p_tag(
+            [r_tag],
+            jc='start',
+        )
+        body = p_tag
+
+        xml = DXB.xml(body)
+        return xml
+
+
+class SuperAndSubScripts(_TranslationTestCase):
+    expected_output = '''
+        <p>AAA<sup>BBB</sup></p>
+        <p><sub>CCC</sub>DDD</p>
+    '''
+
+    def get_xml(self):
+        p_tags = [
+            DXB.p_tag(
+                [
+                    DXB.r_tag([DXB.t_tag('AAA')]),
+                    DXB.r_tag(
+                        [DXB.t_tag('BBB')],
+                        rpr=DXB.rpr_tag({'vertAlign': 'superscript'}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('CCC')],
+                        rpr=DXB.rpr_tag({'vertAlign': 'subscript'}),
+                    ),
+                    DXB.r_tag([DXB.t_tag('DDD')]),
+                ],
+            ),
+        ]
+        body = ''
+        for p_tag in p_tags:
+            body += p_tag
+
+        xml = DXB.xml(body)
+        return xml
+
+
+class AvaliableInlineTags(_TranslationTestCase):
+    expected_output = '''
+        <p><strong>aaa</strong></p>
+        <p><span class="pydocx-underline">bbb</span></p>
+        <p><em>ccc</em></p>
+        <p><span class="pydocx-caps">ddd</span></p>
+        <p><span class="pydocx-small-caps">eee</span></p>
+        <p><span class="pydocx-strike">fff</span></p>
+        <p><span class="pydocx-strike">ggg</span></p>
+        <p><span class="pydocx-hidden">hhh</span></p>
+        <p><span class="pydocx-hidden">iii</span></p>
+        <p><sup>jjj</sup></p>
+    '''
+
+    def get_xml(self):
+        p_tags = [
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('aaa')],
+                        rpr=DXB.rpr_tag({'b': None}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('bbb')],
+                        rpr=DXB.rpr_tag({'u': None}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('ccc')],
+                        rpr=DXB.rpr_tag({'i': None}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('ddd')],
+                        rpr=DXB.rpr_tag({'caps': None}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('eee')],
+                        rpr=DXB.rpr_tag({'smallCaps': None}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('fff')],
+                        rpr=DXB.rpr_tag({'strike': None})
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('ggg')],
+                        rpr=DXB.rpr_tag({'dstrike': None}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('hhh')],
+                        rpr=DXB.rpr_tag({'vanish': None})
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('iii')],
+                        rpr=DXB.rpr_tag({'webHidden': None}),
+                    ),
+                ],
+            ),
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag('jjj')],
+                        rpr=DXB.rpr_tag({'vertAlign': 'superscript'}),
+                    ),
+                ],
+            ),
+        ]
+        body = ''
+        for p_tag in p_tags:
+            body += p_tag
+
+        xml = DXB.xml(body)
+        return xml
+
+
+class UnicodeTestCase(_TranslationTestCase):
+    expected_output = u"""
+        <p>\U0010001f</p>
+    """
+
+    def get_xml(self):
+        tags = [
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag(r'&#x10001F;')],
+                    ),
+                ],
+            ),
+        ]
+
+        body = ''
+        for tag in tags:
+            body += tag
+        xml = DXB.xml(body)
+        return xml.encode('utf-8')
+
+
+class NoTextInTTagTestCase(_TranslationTestCase):
+    expected_output = u"""
+    """
+
+    def get_xml(self):
+        tags = [
+            DXB.p_tag(
+                [
+                    DXB.r_tag(
+                        [DXB.t_tag(None)],
+                    ),
+                ],
+            ),
+        ]
+
+        body = ''
+        for tag in tags:
+            body += tag
+        xml = DXB.xml(body)
+        return xml.encode('utf-8')
